@@ -25,6 +25,7 @@
 #include "atr.h"
 #include "atr_sync.h"
 #include <stdlib.h>
+#include <string.h>
 
 /*
  * Not exported constants definition
@@ -34,6 +35,7 @@
 #define CARDTERMINAL_REQUESTICC_BUFFER_SIZE	35
 #define CARDTERMINAL_GETSTATUS_BUFFER_SIZE	19
 #define CARDTERMINAL_EJECTICC_BUFFER_SIZE	2
+#define CARDTERMINAL_MANUFACTURER		"DETWK"
 
 /* 
  * Not exported functions declaration
@@ -432,7 +434,7 @@ CardTerminal_ResetCT (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  if (CT_Slot_GetICCType (ct->slots[sn]) == CT_SLOT_ICC_ASYNC)
 	    {
 	      if (atr != NULL)
-		ATR_GetRaw (atr, buffer, &length);
+		ATR_GetRaw ((ATR *) atr, buffer, &length);
 	      else 
 		length = 0;
 	      
@@ -443,7 +445,7 @@ CardTerminal_ResetCT (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  else
 	    {
 	      if (atr != NULL)
-		ATR_Sync_GetRaw (atr, buffer, &length);
+		ATR_Sync_GetRaw ((ATR_Sync *) atr, buffer, &length);
 	      else
 		length = 0;
 				    
@@ -461,7 +463,7 @@ CardTerminal_ResetCT (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  if (CT_Slot_GetICCType (ct->slots[sn]) == CT_SLOT_ICC_ASYNC)
 	    {
 	      if (atr != NULL)
-		ATR_GetHistoricalBytes (atr, buffer, &length);
+		ATR_GetHistoricalBytes ((ATR *) atr, buffer, &length);
 	      else
 		length = 0;
 	      
@@ -472,7 +474,7 @@ CardTerminal_ResetCT (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  else
 	    {
 	      if (atr != NULL)
-		ATR_Sync_GetHistoricalBytes (atr, buffer, &length);
+		ATR_Sync_GetHistoricalBytes ((ATR_Sync *) atr, buffer, &length);
 	      else
 		length = 0;
 
@@ -603,7 +605,7 @@ CardTerminal_RequestICC (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  if (CT_Slot_GetICCType (ct->slots[sn]) == CT_SLOT_ICC_ASYNC)
 	    {
 	      if (atr != NULL)
-		ATR_GetRaw (atr, buffer, &length);
+		ATR_GetRaw ((ATR *) atr, buffer, &length);
 	      else
 		length = 0;
 	      
@@ -614,7 +616,7 @@ CardTerminal_RequestICC (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  else
 	    {
 	      if (atr != NULL)
-		ATR_Sync_GetRaw (atr, buffer, &length);
+		ATR_Sync_GetRaw ((ATR_Sync *) atr, buffer, &length);
 	      else
 		length = 0;
 	      
@@ -632,7 +634,7 @@ CardTerminal_RequestICC (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  if (CT_Slot_GetICCType (ct->slots[sn]) == CT_SLOT_ICC_ASYNC)
 	    {
 	      if (atr != NULL)
-		ATR_GetHistoricalBytes (atr, buffer, &length);
+		ATR_GetHistoricalBytes ((ATR *) atr, buffer, &length);
 	      else
 		length = 0;
 
@@ -643,7 +645,7 @@ CardTerminal_RequestICC (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
 	  else
 	    {
 	      if (atr != NULL)
-		ATR_Sync_GetHistoricalBytes (atr, buffer, &length);
+		ATR_Sync_GetHistoricalBytes ((ATR_Sync *) atr, buffer, &length);
 	      else
 		length = 0;
 
@@ -704,7 +706,7 @@ CardTerminal_GetStatus (CardTerminal * ct, APDU_Cmd * cmd, APDU_Rsp ** rsp)
       length = 17;
 
       /* CT Manufacturer */ 
-      memcpy(buffer,PACKAGE,5);
+      memcpy(buffer,CARDTERMINAL_MANUFACTURER,5);
   
       /* CT type */
       if (ct->slots[0] != NULL)
