@@ -299,8 +299,6 @@ IFD_Towitoko_SetBaudrate (IFD * ifd, unsigned long baudrate)
   usleep (150000L);
 #endif
 
-//  IFD_Towitoko_GetReaderInfo (ifd);
-  
   return IFD_TOWITOKO_OK;
 }
 
@@ -768,10 +766,10 @@ extern int IFD_Towitoko_ResetSyncICC (IFD * ifd, ATR_Sync ** atr)
     return IFD_TOWITOKO_IO_ERROR;
 
   if (!IO_Serial_Read (ifd->io, IFD_TOWITOKO_TIMEOUT, 1, status))
-    return IFD_TOWITOKO_IO_ERROR; 
+    return IFD_TOWITOKO_IO_ERROR;
 
   if (status[0] != 0x01)
-    return IFD_TOWITOKO_CHK_ERROR; 
+    return IFD_TOWITOKO_CHK_ERROR;
     
   if (IFD_Towitoko_ReadBuffer (ifd, 8, atr_buffer) != IFD_TOWITOKO_OK)
     return IFD_TOWITOKO_IO_ERROR;
@@ -1512,6 +1510,38 @@ IFD_Towitoko_GetType (IFD * ifd)
   return ifd->type;
 }
 
+void
+IFD_Towitoko_GetDescription (IFD * ifd, BYTE * desc, unsigned length)
+{
+	char buffer[3];
+
+	if (ifd->type == IFD_TOWITOKO_CHIPDRIVE_EXT_II)
+		memcpy (desc,"CE2",MIN(length,3));
+
+	else if  (ifd->type == IFD_TOWITOKO_CHIPDRIVE_EXT_I)
+		memcpy (desc,"CE1",MIN(length,3));
+
+	else if (ifd->type == IFD_TOWITOKO_CHIPDRIVE_INT)
+		memcpy (desc,"CDI",MIN(length,3));
+
+	else if (ifd->type == IFD_TOWITOKO_CHIPDRIVE_MICRO)
+		memcpy (desc,"CDM",MIN(length,3));
+
+	else if (ifd->type == IFD_TOWITOKO_KARTENZWERG_II) 
+		memcpy (desc,"KZ2",MIN(length,3));
+
+	else if (ifd->type == IFD_TOWITOKO_KARTENZWERG)
+		memcpy (desc,"KZ1",MIN(length,3));
+
+	else 
+		memcpy (desc,"UNK",MIN(length,3));
+	
+	snprintf (buffer, 3, "%02X", ifd->firmware);
+
+	if (length > 3)
+		memcpy (desc+3, buffer, MIN(length-3,2));
+}
+
 BYTE
 IFD_Towitoko_GetFirmware (IFD * ifd)
 {
