@@ -39,25 +39,15 @@
 #include <time.h>
 #include "io_serial.h"
 
-/*
- * Not exported macros definition
- */
-
-#define bitrate(a)	((a)==0 ? B0: (a)<=50 ? B50: \
-			(a)<=75 ? B75: (a)<=110 ? B110: \
-			(a)<=134 ? B134: (a)<=150 ? B150: \
-			(a)<=200 ? B200: (a)<=300 ? B300: \
-			(a)<=600 ? B600: (a)<=1200 ? B1200: \
-		       	(a)<=1800 ? B1800: (a)<=2400 ? B2400: \
-			(a)<=4800 ? B4800: (a)<=9600 ? B9600: \
-			(a)<=19200 ? B19200: (a)<=38400 ? B38400: \
-			(a)<=57600 ? B57600: B115200)
-
 #define IO_SERIAL_FILENAME_LENGTH 	32
 
 /*
  * Internal functions declaration
  */
+
+static int
+IO_Serial_Bitrate(int bitrate);
+
 static bool
 IO_Serial_WaitToRead (int hnd, unsigned delay_ms, unsigned timeout_ms);
 
@@ -143,130 +133,208 @@ IO_Serial_GetProperties (IO_Serial * io, IO_Serial_Properties * props)
 
   switch (o_speed)
     {
+#ifdef B0
     case B0:
       props->output_bitrate = 0;
       break;
+#endif
+#ifdef B50
     case B50:
       props->output_bitrate = 50;
       break;
+#endif
+#ifdef B75
     case B75:
       props->output_bitrate = 75;
       break;
+#endif
+#ifdef B110
     case B110:
       props->output_bitrate = 110;
       break;
+#endif
+#ifdef B134
     case B134:
       props->output_bitrate = 134;
       break;
+#endif
+#ifdef B150
     case B150:
       props->output_bitrate = 150;
       break;
+#endif
+#ifdef B200
     case B200:
       props->output_bitrate = 200;
       break;
+#endif
+#ifdef B300
     case B300:
       props->output_bitrate = 300;
       break;
+#endif
+#ifdef B600
     case B600:
       props->output_bitrate = 600;
       break;
+#endif
+#ifdef B1200
     case B1200:
       props->output_bitrate = 1200;
       break;
+#endif
+#ifdef B1800
     case B1800:
       props->output_bitrate = 1800;
       break;
+#endif
+#ifdef B2400
     case B2400:
       props->output_bitrate = 2400;
       break;
+#endif
+#ifdef B4800
     case B4800:
       props->output_bitrate = 4800;
       break;
+#endif
+#ifdef B9600
     case B9600:
       props->output_bitrate = 9600;
       break;
+#endif
+#ifdef B19200
     case B19200:
       props->output_bitrate = 19200;
       break;
+#endif
+#ifdef B38400
     case B38400:
       props->output_bitrate = 38400;
       break;
+#endif
+#ifdef B57600
     case B57600:
       props->output_bitrate = 57600;
       break;
+#endif
+#ifdef B115200
     case B115200:
-      props->output_bitrate = 115200L;
-      break;
-#ifndef OS_CYGWIN32
-    case B230400:
-      props->output_bitrate = 230400L;
+      props->output_bitrate = 115200;
       break;
 #endif
+#ifdef B230400
+    case B230400:
+      props->output_bitrate = 230400;
+      break;
+#endif
+    default:
+      props->output_bitrate = 1200;
+      break;
     }
 
   i_speed = cfgetispeed (&currtio);
 
   switch (i_speed)
     {
+#ifdef B0
     case B0:
       props->input_bitrate = 0;
       break;
+#endif
+#ifdef B50
     case B50:
       props->input_bitrate = 50;
       break;
+#endif
+#ifdef B75
     case B75:
       props->input_bitrate = 75;
       break;
+#endif
+#ifdef B110
     case B110:
       props->input_bitrate = 110;
       break;
+#endif
+#ifdef B134
     case B134:
       props->input_bitrate = 134;
       break;
+#endif
+#ifdef B150
     case B150:
       props->input_bitrate = 150;
       break;
+#endif
+#ifdef B200
     case B200:
       props->input_bitrate = 200;
       break;
+#endif
+#ifdef B300
     case B300:
       props->input_bitrate = 300;
       break;
+#endif
+#ifdef B600
     case B600:
       props->input_bitrate = 600;
       break;
+#endif
+#ifdef B1200
     case B1200:
       props->input_bitrate = 1200;
       break;
+#endif
+#ifdef B1800
     case B1800:
       props->input_bitrate = 1800;
       break;
+#endif
+#ifdef B2400
     case B2400:
       props->input_bitrate = 2400;
       break;
+#endif
+#ifdef B4800
     case B4800:
       props->input_bitrate = 4800;
       break;
+#endif
+#ifdef B9600
     case B9600:
       props->input_bitrate = 9600;
       break;
+#endif
+#ifdef B19200
     case B19200:
       props->input_bitrate = 19200;
       break;
+#endif
+#ifdef B38400
     case B38400:
       props->input_bitrate = 38400;
       break;
+#endif
+#ifdef B57600
     case B57600:
       props->input_bitrate = 57600;
       break;
+#endif
+#ifdef B115200
     case B115200:
-      props->input_bitrate = 115200L;
-      break;
-#ifndef OS_CYGWIN32
-    case B230400:
-      props->input_bitrate = 230400L;
+      props->input_bitrate = 115200;
       break;
 #endif
+#ifdef B230400
+    case B230400:
+      props->input_bitrate = 230400;
+      break;
+#endif
+    default:
+      props->input_bitrate = 1200;
+      break;
     }
 
   switch (currtio.c_cflag & CSIZE)
@@ -298,7 +366,7 @@ IO_Serial_GetProperties (IO_Serial * io, IO_Serial_Properties * props)
   else
     props->stopbits = 1;
 
-#ifndef OS_CYGWIN32
+#if !defined(OS_CYGWIN32) && !defined(OS_HPUX)
   if (ioctl (io->fd, TIOCMGET, &mctl) < 0)
     return FALSE;
 
@@ -329,7 +397,7 @@ IO_Serial_SetProperties (IO_Serial * io, IO_Serial_Properties * props)
   struct termios newtio;
   unsigned int modembits;
 #if 1
-#ifndef OS_CYGWIN32
+#if !defined(OS_CYGWIN32) && !defined(OS_HPUX)
 
   modembits = TIOCM_DTR;
 
@@ -364,9 +432,11 @@ IO_Serial_SetProperties (IO_Serial * io, IO_Serial_Properties * props)
 
   memset (&newtio, 0, sizeof (newtio));
 
-  cfsetispeed (&newtio, bitrate (props->input_bitrate));
-  cfsetospeed (&newtio, bitrate (props->output_bitrate));
+  /* Set the bitrate */
+  cfsetispeed (&newtio, IO_Serial_Bitrate (props->input_bitrate));
+  cfsetospeed (&newtio, IO_Serial_Bitrate (props->output_bitrate));
 
+  /* Set the character size */
   switch (props->bits)
     {
     case 5:
@@ -386,24 +456,25 @@ IO_Serial_SetProperties (IO_Serial * io, IO_Serial_Properties * props)
       break;
     }
 
+  /* Set the parity */
   switch (props->parity)
     {
     case IO_SERIAL_PARITY_ODD:
-      newtio.c_cflag |= PARODD | PARENB;
-      newtio.c_iflag |= INPCK;
+      newtio.c_cflag |= PARENB;
+      newtio.c_cflag |= PARODD;
       break;
 
     case IO_SERIAL_PARITY_EVEN:
-      newtio.c_cflag &= (~PARODD);
+
       newtio.c_cflag |= PARENB;
-      newtio.c_iflag |= INPCK;
+      newtio.c_cflag &= ~PARODD;
       break;
 
     case IO_SERIAL_PARITY_NONE:
-      newtio.c_cflag &= (~PARENB);
-      newtio.c_iflag &= (~INPCK);
+      newtio.c_cflag &= ~PARENB;
     }
 
+  /* Set the number of stop bits */
   switch (props->stopbits)
     {
     case 1:
@@ -414,12 +485,17 @@ IO_Serial_SetProperties (IO_Serial * io, IO_Serial_Properties * props)
       break;
     }
 
+  /* Selects raw (non-canonical) input and output */
+  newtio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
+  newtio.c_oflag &= ~OPOST;
+#if 1
+  /* Ignore parity errors!!! Windows driver does so why shouldn't I? */
+  newtio.c_iflag |= IGNPAR;
+#endif
+  /* Enable receiber, hang on close, ignore control line */
   newtio.c_cflag |= CREAD | HUPCL | CLOCAL;
-  newtio.c_iflag &= ~(IGNPAR | PARMRK | INLCR | IGNCR | ICRNL | ISTRIP);
-  newtio.c_iflag |= BRKINT;
-  newtio.c_lflag &= ~(ICANON | ECHO);
-  newtio.c_oflag = 0;
-  newtio.c_lflag = 0;
+
+  /* Read 1 byte minimun, no timeout specified */
   newtio.c_cc[VMIN] = 1;
   newtio.c_cc[VTIME] = 0;
 
@@ -588,9 +664,72 @@ IO_Serial_Delete (IO_Serial * io)
 }
 
 /*
- * Internal funtions definition
+ * Internal functions definition
  */
 
+static int
+IO_Serial_Bitrate(int bitrate)
+{
+#ifdef B230400
+	if ((bitrate)>=230400) return B230400;
+#endif
+#ifdef B115200
+	if ((bitrate)>=115200) return B115200;
+#endif
+#ifdef B57600
+	if ((bitrate)>=57600) return B57600;
+#endif
+#ifdef B38400
+	if ((bitrate)>=38400) return B38400;
+#endif
+#ifdef B19200
+	if ((bitrate)>=19200) return B19200;
+#endif
+#ifdef B9600
+	if ((bitrate)>=9600) return B9600;
+#endif
+#ifdef B4800
+	if ((bitrate)>=4800) return B4800;
+#endif
+#ifdef B2400
+	if ((bitrate)>=2400) return B2400;
+#endif
+#ifdef B1800
+	if ((bitrate)>=1800) return B1800;
+#endif
+#ifdef B1200
+	if ((bitrate)>=1200) return B1200;
+#endif
+#ifdef B600
+	if ((bitrate)>=600) return B600;
+#endif
+#ifdef B300
+	if ((bitrate)>=300) return B300;
+#endif
+#ifdef B200
+	if ((bitrate)>=200) return B200;
+#endif
+#ifdef B150
+	if ((bitrate)>=150) return B150;
+#endif
+#ifdef B134
+	if ((bitrate)>=134) return B134;
+#endif
+#ifdef B110
+	if ((bitrate)>=110) return B110;
+#endif
+#ifdef B75
+	if ((bitrate)>=75) return B75;
+#endif
+#ifdef B50
+	if ((bitrate)>=50) return B50;
+#endif
+#ifdef B0
+	if ((bitrate)>=0) return B0;
+#endif
+	return 0;	/* Should never get here */
+}
+
 static bool
 IO_Serial_WaitToRead (int hnd, unsigned delay_ms, unsigned timeout_ms)
 {
@@ -793,6 +932,10 @@ IO_Serial_DeviceName (unsigned com, bool usbserial, char * filename, unsigned le
   snprintf (filename, length, "/dev/tty%02d", com);
 #endif /* OS_DIGITAL_UNIX */
 
+#ifdef OS_AIX
+  snprintf (filename, length, "/dev/tty%d", com - 1);
+#endif /* OS_AIX */
+
 #ifdef OS_SCO
   snprintf (filename, length, "/dev/tty%da", com);
 #endif /* OS_SCO */
@@ -802,6 +945,10 @@ IO_Serial_DeviceName (unsigned com, bool usbserial, char * filename, unsigned le
   snprintf (filename, length, "/dev/cuaa%d", com - 1);
 #endif /* OS_FREEBSD */
 
+#ifdef OS_NETBSD
+  snprintf (filename, length, "/dev/ttyC%d", com - 1);
+#endif /* OS_NETBSD */
+
 #ifdef OS_OPENBSD
   /* support for for FreeBSD added by Martin Preuss<libchipcard@aquamaniac.de> */
   snprintf (filename, length, "/dev/cua%02d", com - 1);
