@@ -23,6 +23,7 @@
 #include "defines.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "protocol_t0.h"
 #include "atr.h"
 
@@ -101,7 +102,7 @@ Protocol_T0_Init (Protocol_T0 * t0, ICC_Async * icc, PPS_ProtocolParameters * pa
     wi = PROTOCOL_T0_DEFAULT_WI;
 
   /* WWT = 960 * WI * (Fi / f) * 1000 milliseconds */
-  t0->wwt = 960 * wi * (params->f / ICC_Async_GetClockRate (t0->icc)) * 1000;
+  t0->wwt = (long unsigned int) (960 * wi * (params->f / ICC_Async_GetClockRate (t0->icc)) * 1000);
   
   /* Set timings */
   ICC_Async_GetTimings (t0->icc, &timings);
