@@ -1,5 +1,5 @@
 /*
-    Small tester application that runs against CT-API library.
+    Big bad tester application that runs against CT-API library.
     Handles up to four card-terminal interfaces simultaneously.
     It issues commands for both synchronous and asynchronous cards.
  
@@ -24,37 +24,39 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#if !defined DEBUG_IO && !defined DEBUG_IFD && !defined DEBUG_ICC && !defined DEBUG_PROTOCOL && !defined DEBUG_CTAPI
+/* Define if you want multi-thread. Disabled by default if we are debugging  */
+#define MULTI_THREAD 
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
+#include <time.h>
 #include "ctapi.h"
 #include "ctbcs.h"
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
 #include <pthread.h>
 #endif
-#ifdef OS_FREEBSD
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
-/* Class byte for Cryptoflex */
-/* #define CLASS 0xC0 */
 
-/* Class byte for Cyberflex */
+/* Default Class byte */
 #define CLASS 0x00
 
-/* Class byte for IBM MFC 3.5 smartcards */
-/* #define CLASS 0xA4 */
-
 /* Status of each card-terminal */
 struct
 {
   unsigned short pn;            /* Serial port (1..4), 0 if not assigned */
+  char port[16];		/* Descripton of port, printed in prompt */
   int status;                   /* -1 void, 0 sync, 1 async */
   unsigned char atr[33];        /* ATR bytes */
   unsigned short atr_size;      /* ATR size */
-#ifdef HAVE_PTHREAD_H
+  unsigned char cla;		/* Class byte for commands */
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
   pthread_t thread;             /* Monitoring thread for this ctn */
   pthread_mutex_t mutex;        /* Mutex for accessin this element */
 #endif
@@ -68,6 +70,7 @@ void menu_loop (void);
 void Initialize (void);
 unsigned short Change (unsigned short);
 void Close (unsigned short);
+void SelectClass (unsigned short);
 void SelectFile (unsigned short);
 void GetResponse (unsigned short);
 void UpdateBinary (unsigned short);
@@ -79,7 +82,7 @@ void ChangePin (unsigned short);
 void ReadData (unsigned short);
 void WriteData (unsigned short);
 
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
 /* Asynchronous monitoring thread function */
 void *Monitor (void *);
 #else
@@ -90,6 +93,15 @@ unsigned short GetSmartcard (unsigned short);
 /* Print status */
 void PrintReport (void);
 
+/* Read bus and port */
+unsigned short ReadPort (char port[16]);
+
+/* Calculate length of memory card */
+unsigned GetMemoryLength(unsigned char * atr, unsigned length);
+
+/* Print array of bytes */
+void PrintArray (unsigned char * buffer, unsigned length);
+
 int
 main (int argc, char *argv[])
 {
@@ -100,9 +112,11 @@ main (int argc, char *argv[])
     {
       ct_list[ctn].pn = 0;
       ct_list[ctn].status = -1;
+      ct_list[ctn].cla = CLASS;
+      memset (ct_list[ctn].port, 0, 16);
       memset (ct_list[ctn].atr, 0, 33);
       ct_list[ctn].atr_size = 0;
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
       pthread_mutex_init (&(ct_list[ctn].mutex), NULL);
 #endif
     }
@@ -116,7 +130,7 @@ main (int argc, char *argv[])
       if (ct_list[ctn].pn != 0)
         {
           Close (ctn);
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
           pthread_mutex_destroy (&(ct_list[ctn].mutex));
 #endif
         }
@@ -148,6 +162,7 @@ menu_loop (void)
           /* Processor cards menu */
           if (ct_list[ctn].status == 1)
             {
+              printf ("sc: Select Class Byte (current: %02X)\n", ct_list[ctn].cla);
               printf ("sf: Select File\n");
               printf ("gr: Get Response\n");
               printf ("vk: Verify Key (Cryptoflex)\n");
@@ -170,7 +185,7 @@ menu_loop (void)
       printf ("q: Quit\n");
 
       if (ct_list[ctn].pn != 0)
-        printf ("COM%d$ ", ct_list[ctn].pn);
+        printf ("%s$ ", ct_list[ctn].port);
       else
         printf ("$ ");
 
@@ -180,7 +195,7 @@ menu_loop (void)
 
       /* Convert input to lowercase */
       for (i = 0; option[i]; i++)
-        option[i] = tolower (option[i]);
+        option[i] = tolower ((int) option[i]);
 
       /* Common options */
       if (strcmp (option, "q") == 0)
@@ -198,7 +213,10 @@ menu_loop (void)
       /* Processor card options */
       if (ct_list[ctn].status == 1)
         {
-          if (strcmp (option, "sf") == 0)
+          if (strcmp (option, "sc") == 0)
+            SelectClass (ctn);
+
+	  else if (strcmp (option, "sf") == 0)
             SelectFile (ctn);
 
           else if (strcmp (option, "gr") == 0)
@@ -256,17 +274,25 @@ PrintReport (void)
     {
       if (ct_list[ctn].pn != 0)
         {
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
           pthread_mutex_lock (&(ct_list[ctn].mutex));
 #endif
           printf
            
  ("**********************************************************************\n");
-          printf ("COM%d\n", ct_list[ctn].pn);
-          printf ("Status: %s\n",
+          printf ("%s\n", ct_list[ctn].port);
+
+          printf ("Status: %s",
                   ct_list[ctn].status == 0 ? "Memory smartcard present" :
                   ct_list[ctn].status == 1 ? "Processor smartcard present" :
                   "No smartcard present (type * to refresh)");
+
+	  if (ct_list[ctn].status == 0)
+		  printf (". Memory size: %d bytes\n", 
+		  GetMemoryLength(ct_list[ctn].atr, ct_list[ctn].atr_size));
+	  else
+		  printf("\n");
+
           if (ct_list[ctn].status != -1)
             {
               printf ("ATR: ");
@@ -274,7 +300,7 @@ PrintReport (void)
                 printf ("%02X ", ct_list[ctn].atr[i]);
               printf ("\n");
             }
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
           pthread_mutex_unlock (&(ct_list[ctn].mutex));
 #endif
         }
@@ -285,7 +311,7 @@ PrintReport (void)
  ("**********************************************************************\n");
 }
 
-#ifndef HAVE_PTHREAD_H
+#if !defined HAVE_PTHREAD_H || !defined MULTI_THREAD
 unsigned short
 GetSmartcard (unsigned short ctn)
 {
@@ -471,22 +497,73 @@ Monitor (void *arg)
 }
 #endif
 
+unsigned short
+ReadPort(char port[16])
+{
+  unsigned short pn;
+
+  scanf("%6s", port);
+
+  if(strncasecmp(port,"COM",3) == 0)
+  {
+	  sscanf(port+3,"%hu",&pn);
+	  snprintf(port,16,"COM%hu",pn);
+  }
+
+  else if (strncasecmp(port,"USB",3) == 0)
+  {
+	  sscanf(port+3,"%hu",&pn);
+	  snprintf(port,16,"USB%hu",pn);
+	  pn += 32768;
+  }
+  else
+  {
+	  sscanf(port,"%hu",&pn);
+	  snprintf(port,16,"COM%hu",pn);
+  }
+
+  return pn;
+}
+
+unsigned 
+GetMemoryLength(unsigned char * atr, unsigned length)
+{
+	if (length < 2)
+		return 0;
+
+	/* This line is dedicated to Rene Puls ;-) */
+	return( 1 << (((int)(atr[1] & 120) >> 3)+6) * 1 << (int)(atr[1] & 7) / 8);
+}
+
+void 
+PrintArray (unsigned char * buffer, unsigned length)
+{
+	unsigned i;
+
+	if (length > 16)
+		printf ("\n");
+
+	for (i=0; i<length; i++)
+	{
+		printf ("%02X ", buffer[i]);
+		if (i%16 == 15)
+			printf ("\n");
+	}
+
+	if (i%16 != 0)
+		printf ("\n");
+}
+
 void
 Initialize (void)
 {
   unsigned short int pn, ctn;
+  char port[16];
   char ret;
-  int dummy;
 
-  printf ("Port number [COM>=1, USB>=32769]: ");
-  scanf ("%hu", &pn);
-  dummy = getchar ();
+  printf ("Port (COM[1..n] or USB[1..n]): ");
 
-  if (pn < 1)
-    {
-      printf ("Invalid port number\n");
-      return;
-    }
+  pn = ReadPort(port);
 
   for (ctn = 0; ctn < 4; ctn++)
     {
@@ -505,7 +582,7 @@ Initialize (void)
       return;
     }
 
-  printf ("Initializing terminal at COM%d...\n", pn);
+  printf ("Initializing terminal at %s...\n",port);
 
 #ifndef CTAPI_WIN32_COM
   ret = CT_init (ctn, pn - 1);
@@ -519,9 +596,10 @@ Initialize (void)
       return;
     }
 
+  strncpy(ct_list[ctn].port,port,16);
   ct_list[ctn].pn = pn;
   
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
     {
       unsigned short *arg;
 
@@ -558,7 +636,7 @@ unsigned short
 Change (unsigned short ctn)
 {
   unsigned short pn, i;
-  int dummy;
+  char port[16];
 
   /* Show the list of open ports */
   printf ("Port number (");
@@ -566,7 +644,7 @@ Change (unsigned short ctn)
   for (i = 0; i < 4; i++)
     {
       if (ct_list[i].pn != 0)
-        printf ("%d", ct_list[i].pn);
+        printf ("%s", ct_list[i].port);
       if (i + 1 < 4)
         {
           if (ct_list[i + 1].pn != 0)
@@ -576,8 +654,7 @@ Change (unsigned short ctn)
         printf ("): ");
     }
 
-  scanf ("%hu", &pn);
-  dummy = getchar ();
+  pn = ReadPort(port);
 
   /* Search the ctn */
   for (i = 0; i < 4; i++)
@@ -598,47 +675,70 @@ Change (unsigned short ctn)
 void
 Close (unsigned short ctn)
 {
-  unsigned short pn;
   char ret;
- 
-  pn = ct_list[ctn].pn;
 
-#ifdef HAVE_PTHREAD_H
+  if ((ct_list[ctn].pn == 0) && (ct_list[ctn].status == -1))
+  {
+	  printf ("Port not open\n");
+	  return;
+  }
+
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
   pthread_mutex_lock (&(ct_list[ctn].mutex));
 #endif
   ct_list[ctn].pn = 0;
   ct_list[ctn].status = -1;
   memset (ct_list[ctn].atr, 0, 33);
   ct_list[ctn].atr_size = 0;
-#ifdef HAVE_PTHREAD_H
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
   pthread_mutex_unlock (&(ct_list[ctn].mutex));
   
   printf ("Waiting for terminal monitoring job to stop...\n");
   pthread_join(ct_list[ctn].thread ,NULL);
 #endif
 
-  printf ("Closing terminal at COM%d\n", pn);
+  printf ("Closing terminal at %s\n", ct_list[ctn].port);
 
   ret = CT_close (ctn);
   if (ret != OK)
-    printf ("Error closing terminal at COM%d\n", pn);
+    printf ("Error closing terminal at %s\n", ct_list[ctn].port);
+}
+
+void
+SelectClass (unsigned short ctn)
+{
+  unsigned char buffer[32];
+  int dummy;
+
+  printf ("Class byte (current is %02X): ", ct_list[ctn].cla);
+  scanf ("%X", (unsigned int *) buffer);
+  dummy = getchar ();
+
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
+    pthread_mutex_lock (&(ct_list[ctn].mutex));
+#endif
+  ct_list[ctn].cla = buffer[0];
+#if defined HAVE_PTHREAD_H && defined MULTI_THREAD
+  pthread_mutex_unlock (&(ct_list[ctn].mutex));
+#endif
 }
 
 void
 SelectFile (unsigned short ctn)
 {
-  unsigned char select_file[8] =
-    { CLASS, 0xA4, 0x00, 0x00, 0x02, 0x3f, 0x00, 0x00 };
+  unsigned char select_file[8] = { CLASS, 0xA4, 0x00, 0x00, 0x02, 0x3f, 0x00, 0x00 };
   unsigned char buffer[32];
   unsigned char res[256];
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, dummy;
+  int dummy;
   char ret;
 
+  select_file[0] = ct_list[ctn].cla;
+
   printf ("File ID: ");
-  scanf ("%X %X", buffer, buffer + 1);
+  scanf ("%X %X", (unsigned int *) buffer, (unsigned int *) buffer + 1);
   dummy = getchar ();
 
   select_file[5] = buffer[0];
@@ -647,11 +747,9 @@ SelectFile (unsigned short ctn)
   dad = 0;
   sad = 2;
   lr = 256;
-  printf ("Command: ");
 
-  for (i = 0; i < 8; i++)
-    printf ("%02X ", select_file[i]);
-  printf ("\n");
+  printf ("Command: ");
+  PrintArray (select_file, 8);
 
   ret = CT_data (ctn, &dad, &sad, 8, select_file, &lr, res);
 
@@ -662,10 +760,7 @@ SelectFile (unsigned short ctn)
     }
 
   printf ("Response: ");
-
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
@@ -677,11 +772,13 @@ GetResponse (unsigned short ctn)
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, dummy;
+  int dummy;
   char ret;
 
+  get_response[0] = ct_list[ctn].cla;
+
   printf ("Response size (hexadecimal): ");
-  scanf ("%X", buffer);
+  scanf ("%X", (unsigned int *) buffer);
   dummy = getchar ();
   get_response[4] = buffer[0];
 
@@ -690,9 +787,7 @@ GetResponse (unsigned short ctn)
   lr = 256;
 
   printf ("Command: ");
-  for (i = 0; i < 5; i++)
-    printf ("%02X ", get_response[i]);
-  printf ("\n");
+  PrintArray (get_response, 5);
 
   ret = CT_data (ctn, &dad, &sad, 5, get_response, &lr, res);
 
@@ -703,9 +798,7 @@ GetResponse (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
@@ -717,33 +810,30 @@ UpdateBinary (unsigned short ctn)
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, size, dummy;
+  int size, dummy;
   char ret;
 
+  update_binary[0] =  ct_list[ctn].cla;
+
   printf ("File size (0..255): ");
   scanf ("%d", &size);
   dummy = getchar ();
   update_binary[4] = (unsigned char) (size % 256);
 
   printf ("Data: ");
-  scanf ("%02X", buffer);
+  scanf ("%02X", (unsigned int *) buffer);
   dummy = getchar ();
 
-  for (i = 5; i < (int) update_binary[4] + 5; i++)
-    update_binary[i] = buffer[0];
+  memset(update_binary + 5, buffer[0], update_binary[4]);
 
   dad = 0;
   sad = 2;
   lr = 256;
 
   printf ("Command: ");
-  for (i = 0; i < (int) update_binary[4] + 5; i++)
-    printf ("%02X ", update_binary[i]);
-  printf ("\n");
+  PrintArray (update_binary, update_binary[4] + 5);
 
-  ret =
-    CT_data (ctn, &dad, &sad, (int) update_binary[4] + 5, update_binary, &lr,
-             res);
+  ret = CT_data (ctn, &dad, &sad, (int) update_binary[4] + 5, update_binary, &lr, res);
 
   if (ret != OK)
     {
@@ -752,9 +842,7 @@ UpdateBinary (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
@@ -766,12 +854,14 @@ ReadBinary (unsigned short ctn)
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, dummy;
+  int dummy;
   char ret;
 
+  read_binary[0] = ct_list[ctn].cla;
+
   /* Read binary */
   printf ("File size: ");
-  scanf ("%X", buffer);
+  scanf ("%X", (unsigned int *) buffer);
   dummy = getchar ();
   read_binary[4] = buffer[0];
 
@@ -780,9 +870,7 @@ ReadBinary (unsigned short ctn)
   lr = 258;
 
   printf ("Command: ");
-  for (i = 0; i < 5; i++)
-    printf ("%02X ", read_binary[i]);
-  printf ("\n");
+  PrintArray (read_binary, 5);
 
   ret = CT_data (ctn, &dad, &sad, 5, read_binary, &lr, res);
 
@@ -793,32 +881,27 @@ ReadBinary (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
 VerifyKey (unsigned short ctn)
 {
   unsigned char verify_key[13] =
-    { 0xF0, 0x2A, 0x00, 0x01, 0x08, 0x47, 0x46, 0x58, 0x49, 0x32, 0x56, 0x78,
-    0x40
-  };
+    { CLASS, 0x2A, 0x00, 0x01, 0x08, 0x47, 0x46, 0x58, 0x49, 0x32, 0x56, 0x78, 0x40 };
   unsigned char res[256];
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i;
   char ret;
 
+  verify_key[0] = ct_list[ctn].cla;
+
   dad = 0;
   sad = 2;
   lr = 256;
   printf ("Command: ");
-  for (i = 0; i < 13; i++)
-    printf ("%02X ", verify_key[i]);
-  printf ("\n");
+  PrintArray (verify_key, 13);
 
   ret = CT_data (ctn, &dad, &sad, 13, verify_key, &lr, res);
 
@@ -829,10 +912,7 @@ VerifyKey (unsigned short ctn)
     }
 
   printf ("Response: ");
-
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
@@ -842,11 +922,11 @@ SendPPS (unsigned short ctn)
   unsigned char buffer[32], res[256];
   unsigned char dad, sad;
   unsigned short lr;
-  int dummy, i;
+  int dummy;
   char ret;
 
   printf ("PPS request (PPSS PPS0 PPS1): ");
-  scanf ("%X %X %X", buffer, buffer + 1, buffer + 2);
+  scanf ("%X %X %X", (unsigned int *) buffer, (unsigned int *) buffer + 1, (unsigned int *) buffer + 2);
   dummy = getchar ();
 
   reset[5] = buffer[0];
@@ -858,9 +938,7 @@ SendPPS (unsigned short ctn)
   lr = 256;
 
   printf ("Command: ");
-  for (i = 0; i < 8; i++)
-    printf ("%02X ", reset[i]);
-  printf ("\n");
+  PrintArray (reset, 8);
 
   ret = CT_data (ctn, &dad, &sad, 9, reset, &lr, res);
 
@@ -871,27 +949,23 @@ SendPPS (unsigned short ctn)
     }
 
   printf ("Response: ");
-
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
 EnterPin (unsigned short ctn)
 {
-  unsigned char verify[8] =
-    { 0x00, 0x20, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00 };
+  unsigned char verify[8] = { 0x00, 0x20, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00 };
   unsigned char buffer[32];
   unsigned char res[256];
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, dummy;
+  int dummy;
   char ret;
 
   printf ("PIN (3 bytes): ");
-  scanf ("%X %X %X", buffer, buffer + 1, buffer + 2);
+  scanf ("%X %X %X", (unsigned int *) buffer, (unsigned int *) buffer + 1, (unsigned int *) buffer + 2);
   dummy = getchar ();
 
   verify[5] = buffer[0];
@@ -903,9 +977,7 @@ EnterPin (unsigned short ctn)
   lr = 256;
 
   printf ("Command: ");
-  for (i = 0; i < 8; i++)
-    printf ("%02X ", verify[i]);
-  printf ("\n");
+  PrintArray (verify, 8);
 
   ret = CT_data (ctn, &dad, &sad, 8, verify, &lr, res);
 
@@ -916,10 +988,7 @@ EnterPin (unsigned short ctn)
     }
 
   printf ("Response: ");
-
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
@@ -932,11 +1001,11 @@ ChangePin (unsigned short ctn)
   unsigned char dad;
   unsigned char sad;
   unsigned short lr;
-  int i, dummy;
+  int dummy;
   char ret;
 
   printf ("PIN (3 bytes): ");
-  scanf ("%X %X %X", buffer, buffer + 1, buffer + 2);
+  scanf ("%X %X %X", (unsigned int *) buffer, (unsigned int *) buffer + 1, (unsigned int *) buffer + 2);
   dummy = getchar ();
 
   change[5] = buffer[0];
@@ -944,7 +1013,7 @@ ChangePin (unsigned short ctn)
   change[7] = buffer[2];
 
   printf ("New PIN (3 bytes): ");
-  scanf ("%X %X %X", buffer, buffer + 1, buffer + 2);
+  scanf ("%X %X %X", (unsigned int *) buffer, (unsigned int *) buffer + 1, (unsigned int *) buffer + 2);
   dummy = getchar ();
 
   change[8] = buffer[0];
@@ -956,9 +1025,7 @@ ChangePin (unsigned short ctn)
   lr = 256;
 
   printf ("Command: ");
-  for (i = 0; i < 11; i++)
-    printf ("%02X ", change[i]);
-  printf ("\n");
+  PrintArray (change, 11);
 
   ret = CT_data (ctn, &dad, &sad, 11, change, &lr, res);
 
@@ -969,24 +1036,21 @@ ChangePin (unsigned short ctn)
     }
 
   printf ("Response: ");
-
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
 void
 ReadData (unsigned short ctn)
 {
   unsigned char select_file[7] = { 0x00, 0xA4, 0x00, 0x00, 0x02, 0x3F, 0x00 };
-  unsigned char read_binary[5] = { 0x00, 0xB0, 0x00, 0x00, 0x00 };
+  unsigned char read_binary[6] = { 0x00, 0xB0, 0x00, 0x00, 0x00, 0x00 };
   int address;
-  int size;
-  unsigned char res[258];
+  int size, total_size;
+  unsigned char * res;
   unsigned char dad;
   unsigned char sad;
-  unsigned short lr;
-  int i, dummy;
+  unsigned short lr, lc;
+  int dummy;
   char ret;
 
   printf ("Address: ");
@@ -996,20 +1060,31 @@ ReadData (unsigned short ctn)
   read_binary[2] = (unsigned char) (address >> 8);
   read_binary[3] = (unsigned char) (address & 0x00FF);
 
-  printf ("Size (0..255): ");
+  total_size  = GetMemoryLength(ct_list[ctn].atr, ct_list[ctn].atr_size);
+
+  printf ("Size (0..%d): ", total_size);
   scanf ("%d", &size);
   dummy = getchar ();
 
-  read_binary[4] = (unsigned char) size;
+  if (size < 256)
+  {
+	read_binary[4] = (unsigned char) size;
+	lc = 5;
+  }
+  else
+  {
+	read_binary[4] = 0;
+	read_binary[5] = (unsigned char) (size >> 8);
+	read_binary[6] = (unsigned char) (size & 0x00FF);
+	lc = 7;
+  }
 
   printf ("Command: ");
-  for (i = 0; i < 7; i++)
-    printf ("%02X ", select_file[i]);
-  printf ("\n");
+  PrintArray (select_file, 7);
 
   dad = 0;
   sad = 2;
-  lr = 258;
+  res  = calloc ((lr = 2), sizeof (unsigned char));
 
   /* Select MF */
   ret = CT_data (ctn, &dad, &sad, 7, select_file, &lr, res);
@@ -1021,21 +1096,25 @@ ReadData (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
+
+  free (res);
 
   printf ("Command: ");
-  for (i = 0; i < 5; i++)
-    printf ("%02X ", read_binary[i]);
-  printf ("\n");
+  PrintArray (read_binary, lc);
 
   dad = 0;
   sad = 2;
-  lr = 258;
+  if (size != 0)
+  	res = calloc ((lr = size + 2), sizeof (unsigned char));
+  else
+	res  = calloc ((lr = total_size + 2), sizeof (unsigned char));
+
+  if (res == NULL)
+	  return;
 
   /* Read binary */
-  ret = CT_data (ctn, &dad, &sad, 5, read_binary, &lr, res);
+  ret = CT_data (ctn, &dad, &sad, lc, read_binary, &lr, res);
 
   if (ret != OK)
     {
@@ -1044,50 +1123,75 @@ ReadData (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
+
+  free  (res);
 }
 
 void
 WriteData (unsigned short ctn)
 {
   unsigned char select_file[7] = { 0x00, 0xA4, 0x00, 0x00, 0x02, 0x3F, 0x00 };
-  unsigned char update_binary[260] = { 0x00, 0xD6, 0x00, 0x00, 0x00 };
+  unsigned char * update_binary;
   int address;
-  int size;
+  int size, total_size;
   unsigned char buffer[32];
   unsigned char res[256];
   unsigned char dad;
   unsigned char sad;
-  unsigned short lr;
-  int i, dummy;
+  unsigned short lr, lc;
+  int dummy;
   char ret;
 
   printf ("Address: ");
   scanf ("%d", &address);
   dummy = getchar ();
 
-  update_binary[2] = (unsigned char) (address >> 8);
-  update_binary[3] = (unsigned char) (address & 0x00FF);
+  total_size  = GetMemoryLength(ct_list[ctn].atr, ct_list[ctn].atr_size);
 
-  printf ("Size (0..255): ");
+  printf ("Size (0..%d): ", total_size);
   scanf ("%d", &size);
   dummy = getchar ();
 
-  update_binary[4] = (unsigned char) (size % 256);
-
   printf ("Data: ");
-  scanf ("%X", buffer);
+  scanf ("%X", (unsigned int *) buffer);
   dummy = getchar ();
 
-  for (i = 5; i < update_binary[4] + 5; i++)
-    update_binary[i] = buffer[0];
+  if  (size < 256)
+  {
+	update_binary = calloc (size +5, sizeof (unsigned char));
+
+	if (update_binary == NULL)
+		return;
+
+	update_binary[0] = 0x00;
+	update_binary[1] = 0xD6;
+	update_binary[2] = (unsigned char) (address >> 8);
+	update_binary[3] = (unsigned char) (address & 0x00FF);
+	update_binary[4] = size;
+	memset (update_binary+5, buffer[0], size);
+	lc = size + 5;
+  }
+  else
+  {
+	update_binary = calloc (size +7, sizeof (unsigned char));
+
+	if (update_binary == NULL)
+		return;
+
+	update_binary[0] = 0x00;
+	update_binary[1] = 0xD6;
+	update_binary[2] = (unsigned char) (address >> 8);
+	update_binary[3] = (unsigned char) (address & 0x00FF);
+	update_binary[4] = 0;
+	update_binary[5] = (unsigned char) (size >> 8);
+	update_binary[6] = (unsigned char) (size & 0x00FF);
+	memset (update_binary+7, buffer[0], size);
+	lc = size +7;
+  }
 
   printf ("Command: ");
-  for (i = 0; i < 7; i++)
-    printf ("%02X ", select_file[i]);
-  printf ("\n");
+  PrintArray (select_file, 7);
 
   dad = 0;
   sad = 2;
@@ -1099,26 +1203,24 @@ WriteData (unsigned short ctn)
   if (ret != OK)
     {
       fprintf (stderr, "Error on SELECT FILE: %d\n", ret);
+      free (update_binary);
       return;
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 
   printf ("Command: ");
-  for (i = 0; i < update_binary[4] + 5; i++)
-    printf ("%02X ", update_binary[i]);
-  printf ("\n");
+  PrintArray (update_binary, lc);
 
   dad = 0;
   sad = 2;
   lr = 256;
 
   /* Update binary */
-  ret =
-    CT_data (ctn, &dad, &sad, update_binary[4] + 5, update_binary, &lr, res);
+  ret = CT_data (ctn, &dad, &sad, lc, update_binary, &lr, res);
+
+  free (update_binary);
 
   if (ret != OK)
     {
@@ -1127,8 +1229,6 @@ WriteData (unsigned short ctn)
     }
 
   printf ("Response: ");
-  for (i = 0; i < lr; i++)
-    printf ("%02X ", res[i]);
-  printf ("\n");
+  PrintArray (res, lr);
 }
 
